%% num_deriv_plot_comparison_subplot
% Ivan LIEBGOTT @ Novembre 2019
% Modlisation et Simulation des Systmes Multi-Physiques
% avec MATLAB - Simulink
%
% Calcul de la drive numrique d'une fonction en utilisant diffrentes
% methodes de calcul:
% - diffrence finie centre
% - diffrence finie rtrograde
% - diffrence finie progressive
% Une comparaison est effectue avec le calcul de la drive thorique
% exacte effectue en calcul symbolique
% Il est possible de faire varier l'intervalle [a;b] d'tude de la fonction
% Reprsentation de 4 courbes correspondant  4 valeurs du nombres de
% points calculs choisies quitablement rparties entre nb_points_min et
% nb_points_max
% La fonction  driver est stocke dans la fonction f1.m
%% Script
% dfinition de l'intervalle d'tude  [a;b] de la fonction considre
a = 0;
b = 10;

% spcification des nombres de points mini et maxi
nb_points_min = 10;
nb_points_max = 50;
pas = round((nb_points_max - nb_points_min) / 3);
%% 
fig = figure('Units','centimeters','Position',[2 5 35 25]);
set(fig,'Color','white');
sgtitle({'Comparaison des diffrentes mthodes de drivation numrique';...
    ' de la fonction f1 en fonction du nombre de points calculs'});

% initialisation du compteur de boucle
k = 1;

for nb_points = nb_points_min : pas : nb_points_max;
    
subplot(2,2,k); hold on; grid on; grid minor;

% calcul et trac de la drive exacte en calcul symbolique
[t_ana_symbolique,df_num_ana_symbolique] = fderiv_ana_symbolique(a,b,nb_points,@f1);
plot(t_ana_symbolique,df_num_ana_symbolique,'red','LineWidth',2);

% calcul  et trac de la drive avec diffrence finie centre
[t_center,df_num_center] = fderiv_center(a,b,nb_points,@f1);
plot(t_center,df_num_center,'green','LineWidth',2);

% calcul et trac de la drive avec diffrence finie progressive
[t_forward,df_num_forward] = fderiv_forward(a,b,nb_points,@f1);
plot(t_forward,df_num_forward,'blue','LineWidth',2);

% calcul et trac de la drive avec diffrence finie retrograde
[t_backward,df_num_backward] = fderiv_backward(a,b,nb_points,@f1);
plot(t_backward,df_num_backward,'cyan','LineWidth',1);

title('Nombre de points calculs  ' + string(nb_points))
k = k + 1;
end

% spcification de la lgende gnrale du graphique
leg = legend('drive analytique','diffrence finie centre',...
    'diffrence finie progressive','diffrence fine rtrograde');

% Positionnement de la lgende au centre du graphique
newPosition = [0.45 0.47 0.1 0.07];
newUnits = 'normalized';
set(leg,'Position', newPosition,'Units', newUnits);




    


 






