%% num_deriv_plot_error
% Ivan LIEBGOTT @ Novembre 2019
% Modlisation et Simulation des Systmes Multi-Physiques
% avec MATLAB - Simulink
%
% Calcul de la forme mathmathique de l'erreur effectue lors des 
% oprations de drivation numriques pour diffrentes mthodes de calcul:
% - diffrence finie centre
% - diffrence finie rtrograde
% - diffrence finie progressive
% Une comparaison est effectue avec le calcul de la drive thorique
% exacte effectue en calcul symbolique
% 
% On se place arbitrairement au centre de l'intervalle d'tude de la
% fonction en un point x0 afin d'valuer l'erreur en ce point pour 
% diffrentes valeurs de h
% et du nombre de points de calcul
% on utilise ensuite une interpolation polynmiale pour dterminer la forme
% mathmatique de l'erreur en fonction de h
% La fonction  driver est stocke dans la fonction f1.m

%% Script
% dfinition de l'intervalle d'tude [a;b] de la fonction considre
a = 0;
b = 10;

% initialisation du compteur qui servira d'indice
k = 1;

%% On dfinit ici une boucle qui permettra de calculer l'erreur pour un
% chantillon de points allant de 100 points jusqu' 1000 points par pas 
% de 100 pour toutes les mthodes de drivation

for nb_points = 100:100:1000
    
% calcul de la drive exacte en utilisant le calcul symbolique
[t_ana_symbolique,df_num_ana_symbolique] = fderiv_ana_symbolique(a,b,nb_points,@f1);

% calcul de la drive avec diffrence finie centre
[t_center,df_num_center] = fderiv_center(a,b,nb_points,@f1);

% calcul  et trac de la drive avec diffrence finie rtrograde
[t_backward,df_num_backward] = fderiv_backward(a,b,nb_points,@f1);

% calcul  et trac de la drive avec diffrence finie progressive
[t_forward,df_num_forward] = fderiv_forward(a,b,nb_points,@f1);

% calcul des erreurs au point x0, milieu de l'intervalle d'tude en prenant
% comme rfrence la drive exacte dtermine en calcul symbolique
x0 = round(nb_points/2);
error_f_center_h(k) = df_num_ana_symbolique(x0) - df_num_center(x0);
error_f_forward_h(k) = df_num_ana_symbolique(x0) - df_num_forward(x0);
error_f_backward_h(k) = df_num_ana_symbolique(x0) - df_num_backward(x0);

% dtermination de la valeur du pas h(k) pour l'itration en cours
h(k) = (b-a)/nb_points;

% incrmentation de k
k = k + 1;
end

%% Trac et interpolation de l'erreur pour la diffrence finie centre
% Cration d'une nouvelle figure pour les graphiques
figure('Units','centimeters','Position',[2 8 18 12]);
hold all;grid on; grid minor;
plot(h,error_f_center_h,'sm','MarkerSize',10,'LineWidth',2);

% Calcul des coefficients du polynme d'interpolation
coeff_poly_inter_center = polyfit (h,error_f_center_h,2);

% Trac du polynme d'interpolation
t = linspace(0,h(1),100);
poly_inter_center = polyval(coeff_poly_inter_center,t);
plot(t,poly_inter_center,'LineWidth',2);

% Ecriture de la lgende
legend('diffrence finie centre',...
     string(coeff_poly_inter_center(3)) + '  h^2 + ' ...
     + string(coeff_poly_inter_center(2)) + '  h + ' ...
     + string(coeff_poly_inter_center(1)),...
     'Location','northwest','FontSize',12);
 
% Ecriture du titre et des tiquettes des axes
title({'Variation de l''erreur en fonction du pas h' ;...
    'pour la diffrence finie centre'});

xlabel('pas h');
ylabel('erreur');

%% Trac et interpolation de l'erreur pour la diffrence finie progressive
figure('Units','centimeters','Position',[22 8 18 12]);
hold all;grid on; grid minor;
plot(h,error_f_forward_h,'sb','MarkerSize',10,'LineWidth',2)

% Dtermination du polynome d'interpolation pour la diffrence finie
% progressive

% Calcul des coefficients du polynme d'interpolation
coeff_poly_inter_forward = polyfit (h,error_f_forward_h,1);

% Trac du polynme d'interpolation
t = linspace(0,h(1),100);
poly_inter_forward = polyval(coeff_poly_inter_forward,t);
plot(t,poly_inter_forward,'LineWidth',2);

%% Trac et interpolation de l'erreur pour la diffrence finie rtrograde
plot(h,error_f_backward_h,'og','MarkerSize',10,'LineWidth',2)

% Dtermination du polynome d'interpolation pour la diffrence finie
% rtrograde

% Calcul des coefficients du polynme d'interpolation
coeff_poly_inter_backward = polyfit (h,error_f_backward_h,1);

% Trac du polynme d'interpolation
t = linspace(0,h(1),100);
poly_inter_backward = polyval(coeff_poly_inter_backward,t);
plot(t,poly_inter_backward,'LineWidth',2);

% criture de la lgende
legend('diffrence finie progressive',...
       string(coeff_poly_inter_forward(1)) + '  h + ' ...
     + string(coeff_poly_inter_forward(2)) ,...
       'diffrence finie rtrograde',...
     string(coeff_poly_inter_backward(1)) + '  h + ' ...
     + string(coeff_poly_inter_backward(2)),...
     'Location','northwest','FontSize',12);
 
% titre et tiquette des axes
title({'Variation de l''erreur en fonction du pas h' ;...
    'pour les diffrences finies progressive et retrograde'});

xlabel('pas h');
ylabel('erreur');
