%% Loi_en_trapeze_de_vitesse.m
% Ivan LIEBGOTT @ dcembre 2019
% Modlisation et Simulation des Systmes Multi-Physiques
% avec MATLAB - Simulink
% Calcul l'intgrale d'une fonction par la mthode des trapzes
% Un relev exprimental donne l'acclration en mm/s2 en fonction du
% temps. En utilisant l'intgration par la mthode des trapzes, on calcule
% la vitesse et la position.
% les trois courbes sont traces dans la mme fentre graphique

% chargement des mesures de l'acclration
clear all;
load loi_en_trapeze_de_vitesse_data.mat

%% Calcul de la vitesse par intgration de l'acceleration

% dfinition des conditions initiale de la vitesse
vitesse(1) = 0;

% La boucle va ajouter l'aire des trapzes pour chaque intervalle 
% d'amplitude h = t(k-1)-t(k) 
for k = [2:1:length(t)]
    % calcul du pas d'intgration
    h = t(k) - t(k-1);
    
    % construction du vecteur contenant les valeur de la vitesse en
    % utilisant la mthode des trapzes
    vitesse(k) = vitesse(k-1) + (acceleration(k)+acceleration(k-1)) * h/2;
end

%% Calcul de la position par intgration de la vitesse

% dfinition des conditions initiale de la vitesse
position(1) = 0;

% La boucle va ajouter l'aire des trapzes pour chaque intervalle 
% d'amplitude h = t(k-1)-t(k) 
for k = [2:1:length(t)]
    % calcul du pas d'intgration
    h = t(k) - t(k-1);
    
    % construction du vecteur contenant les valeur de la vitesse en
    % utilisant la mthode des trapzes
    position(k) = position(k-1) + (vitesse(k)+vitesse(k-1)) * h/2;
end

% Trac des trois courbes dans la mme fentre graphique
figure('Units','centimeters','Position',[2 8 25 20]);
subplot(3,1,1);
plot(t,acceleration);
grid on; grid minor;
xlabel('temps en s');
ylabel('acclration en mm/s^2')
title('Mesure de l''acclration du chariot en fonction du temps en mm/s^2')

subplot(3,1,2);
plot(t,vitesse);
grid on; grid minor;
xlabel('temps en s');
ylabel('vitesse en mm/s')
title({'Vitesse du chariot en fonction du temps en mm/s',...
    'obtenue par intgration numrique'})

subplot(3,1,3);
plot(t,position);
grid on; grid minor;
xlabel('temps en s');
ylabel('Position en mm')
title({'Position du chariot en fonction du temps en mm',...
    'obtenue par intgration numrique'});

xlabel('temps en s');
ylabel('Position en mm')
title({'Position du chariot en fonction du temps en mm',...
    'obtenue par intgration numrique'});

