%% ode_solve_mass_spring_damper.m
% Ivan LIEBGOTT @ Dcembre 2019
% Modlisation et Simulation des Systmes Multi-Physiques
% avec MATLAB - Simulink
% Permet la rsolution de l'quation diffrentielle de l'oscillateur
% mcanique en utilisant my_ode et ode23
% dfinition de l'intervalle d'tude  [a;b] de la fonction considre
a = 0;
b = 5;

% spcification du nombre de points
nb_points = 5000;

h= (b-a) / nb_points;

%% Rsolution d'une quation diffrentielle du second ordre en utilisant 
% des solveurs numriques

% spcification des conditions initiales CI = [y0 ; dydt0] (matrice 2-by-1)
CI = [0.1;0];

% Rsolution avec my_ode
[t_my_ode,sol_my_ode] = my_ode(@F_mass_spring_damper,a,b,CI,nb_points);

% Rsolution avec ode23
% pour utiliser ode23 on rentre directement en argument le vecteur temporel
% ncssaire au calcul (tspan). Dans my_ode, il fallait rentrer [a;b] et le nombre
% de points ce qui revient exactement au mme
tspan = [a:h:b];
[t_ode23_2,sol_ode23_2] = ode23(@F_mass_spring_damper,tspan,CI);

%% Rsolution analytique en calcul symbolique
%dfinition des variables et de la fonction symbolique
syms t;
syms y(t);

% dfinition des grandeurs physique qui interviennent dans l'quation
m = 1;     % mass : kg
k = 100;   % spring rate : N/m 
b = 2;     % damping coefficient : N/(m/s)
g = 9.81;  % gravity acceleration : N/kg

% dfinition des drives symboliques successives de f(t)
D1y=diff(y,1);
D2y=diff(y,2);

% Construction de l'quation  rsoudre et rsolution  l'aide de la 
% fonction dsolve et spcification des conditions initiales
equ= m*D2y + b*D1y + k*y ==m*g;
sol_symbolique(t)=dsolve(equ,y(0)==CI(1),D1y(0)==CI(2));

% calcul des valeurs du vecteur sol_ana qui contient les valeurs de la
% solution calcule analytiquement

t_ana = [a:h:b];
sol_ana = sol_symbolique(t_ana);

% conversion de la fonction symbolique en double prcision
sol_ana = double(sol_ana);

%reprsentation graphique de la solution
figure;
hold all;
% Trac de la premire colonne de la matrice sol qui correcspond  la
% fonction y, solution de l'quation diffrentielle
plot(t_my_ode,sol_my_ode(:,1),'b','LineWidth',2);
plot(t_ode23_2,sol_ode23_2(:,1),'k--','LineWidth',2);
plot(t_ana,sol_ana,'r');
grid on; grid minor;
title({'Rsolution de l''quation diffrentielle de l''oscillateur mcanique',...
    'en utilisant 3 mthodes diffrentes',string(nb_points) + ' points sont calculs '});

legend('Solution calcule avec my ode','Solution calcule avec ode23',...
    'Solution calcule en symbolique');