%% solve_ode_Van_der_Pol
% Ivan LIEBGOTT @ Dcembre 2019
% Modlisation et Simulation des Systmes Multi-Physiques
% avec MATLAB - Simulink
% comparaison de my_ode et ode_23 pour la rsolution de l'quation
% diffrentielle de Van der Pol (@ Mathworks documentation)
% La fonction qui dfinit la fonction F(t,y) est dfinie en fin de script

% dfinition de l'intervalle d'tude  [a;b] de la fonction considre
a = 0;
b = 40;

% spcification du nombre de points et du pas h
nb_points = 10000;
h= (b-a) / nb_points;

%% Rsolution de l'quation diffrentielle de Van der Pol 

% spcification des conditions initiales CI = [y0 ; dydt0] (matrice 2-by-1)
CI = [2;0];

% Rsolution avec my_ode
[t_my_ode_vdp,sol_my_ode_vdp] = my_ode(@vdp,a,b,CI,nb_points);

% Rsolution avec ode23
% pour utiliser ode23 on rentre directement en argument le vecteur temporel
% ncssaire au calcul (tspan). Dans my_ode, il fallait rentrer [a;b] et le nombre
% de points ce qui revient exactement au mme
tspan = [a:h:b];
[t_ode23_vdp,sol_ode23_vdp] = ode23(@vdp,[a b],CI);

%reprsentation graphique de la solution
figure;
hold all;
% Trac de la premire colonne de la matrice sol qui correspond  la
% fonction y, solution de l'quation diffrentielle
plot(t_my_ode_vdp,sol_my_ode_vdp(:,1),'c','LineWidth',2);
plot(t_ode23_vdp,sol_ode23_vdp(:,1),'k--','LineWidth',2);

grid on; grid minor;
title({'Comparaison des solveurs pour la rsolution',...
    'd''une quation diffrentielle du troisime ordre',string(nb_points) + ' points sont calculs '});

legend('Solution calcule avec my ode','Solution calcule avec ode23');

function f = vdp(t,y)
f = [y(2); (1-y(1)^2)*y(2)-y(1)];
end
